@@ -24,7 +24,6 @@
 
 #include <string.h>
 
-#include "loader.h"
 #include "debug_utils.h"
 #include "unknown_function_handling.h"
 #include "wsi.h"
@@ -263,7 +262,7 @@ void *trampoline_get_proc_addr(struct loader_instance *inst, const char *funcNam
     if (NULL != addr) return addr;
 
     // Unknown device extensions
-    addr = loader_dev_ext_gpa(inst, funcName);
+    addr = loader_dev_ext_gpa_tramp(inst, funcName);
     return addr;
 }
 
@@ -278,17 +277,3 @@ void *globalGetProcAddr(const char *name) {
 
     return NULL;
 }
-
-void *loader_non_passthrough_gdpa(const char *name) {
-    if (!name || name[0] != 'v' || name[1] != 'k') return NULL;
-
-    name += 2;
-
-    if (!strcmp(name, "GetDeviceProcAddr")) return vkGetDeviceProcAddr;
-    if (!strcmp(name, "DestroyDevice")) return vkDestroyDevice;
-    if (!strcmp(name, "GetDeviceQueue")) return vkGetDeviceQueue;
-    if (!strcmp(name, "GetDeviceQueue2")) return vkGetDeviceQueue2;
-    if (!strcmp(name, "AllocateCommandBuffers")) return vkAllocateCommandBuffers;
-
-    return NULL;
-}
